"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutomaticPropsType = void 0;
const tslib_1 = require("tslib");
const plugin_compiler_alipay_1 = require("@morjs/plugin-compiler-alipay");
const utils_1 = require("@morjs/utils");
const path = tslib_1.__importStar(require("path"));
const constants_1 = require("../constants");
class AutomaticPropsType {
    constructor() {
        this.name = 'AutomaticPropsType';
        this.componentsUsedMap = {};
    }
    apply(runner) {
        runner.hooks.beforeRun.tap(this.name, () => {
            // 非 compile 命令 跳过
            if (runner.commandName !== constants_1.COMPILE_COMMAND_NAME)
                return;
            const { sourceType, target } = runner.userConfig;
            const isAlipaySimilarTarget = (0, plugin_compiler_alipay_1.isSimilarTarget)(target);
            // 仅当 js 是 支付宝 DSL 且 编译目标不是 类支付宝小程序 时才执行后续逻辑
            if (sourceType !== utils_1.SourceTypes.alipay)
                return;
            if (sourceType === target)
                return;
            if (isAlipaySimilarTarget)
                return;
            runner.hooks.afterBuildEntries.tapPromise(this.name, async (entries, entryBuilder) => {
                var _a;
                for await (const [key, value] of ((_a = entryBuilder === null || entryBuilder === void 0 ? void 0 : entryBuilder.componentsUsedMap) === null || _a === void 0 ? void 0 : _a.entries()) || []) {
                    value.map(async (item) => {
                        try {
                            const sourceText = await utils_1.fsExtra.readFile(`${path.parse(item.fullPath).dir}/${path.parse(item.fullPath).name}.axml`, 'utf-8');
                            if (sourceText.includes(item.name)) {
                                (0, utils_1.posthtml)()
                                    .use((tree) => {
                                    tree.match({ tag: item.name }, (node) => {
                                        // 保存组件的所有 props 参数，key 为组件 fullPath，value 为所有参数的键值对
                                        this.componentsUsedMap[key] = {
                                            ...(this.componentsUsedMap[key] || {}),
                                            ...((node === null || node === void 0 ? void 0 : node.attrs) || {})
                                        };
                                        return node;
                                    });
                                })
                                    .process(sourceText)
                                    .then((res) => res);
                            }
                        }
                        catch (e) { }
                    });
                }
                return entries;
            });
            // 给组件的 js 文件自动注入缺少的 props
            runner.hooks.scriptParser.tap(this.name, (transformers, options) => {
                var _a, _b, _c, _d, _e;
                if (((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryType) === utils_1.EntryType.component &&
                    !utils_1.lodash.isEmpty((_b = this.componentsUsedMap) === null || _b === void 0 ? void 0 : _b[(_c = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _c === void 0 ? void 0 : _c.path])) {
                    const componentProps = (_d = this.componentsUsedMap) === null || _d === void 0 ? void 0 : _d[(_e = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _e === void 0 ? void 0 : _e.path];
                    ((transformers || {}).before || []).push((0, utils_1.tsTransformerFactory)((node, context) => {
                        const factory = context.factory;
                        if (utils_1.typescript.isPropertyAssignment(node) &&
                            utils_1.typescript.isIdentifier(node.name) &&
                            node.name.escapedText === 'props' &&
                            utils_1.typescript.isObjectLiteralExpression(node.initializer)) {
                            const nodePropsList = node.initializer.properties;
                            // 删除组件已有的 props 项
                            for (const item of nodePropsList) {
                                if (utils_1.typescript.isPropertyAssignment(item) &&
                                    utils_1.typescript.isIdentifier(item.name)) {
                                    delete componentProps[item.name.escapedText];
                                }
                            }
                            const args = [...nodePropsList];
                            for (const key in componentProps) {
                                if (/^\{\{(.+?)\}\}/g.test(componentProps[key])) {
                                    // 补全缺少的 props 项
                                    args.push(factory.createPropertyAssignment(key, factory.createNull()));
                                }
                            }
                            return factory.createPropertyAssignment(node.name, factory.updateObjectLiteralExpression(node.initializer, args));
                        }
                        return node;
                    }));
                    return transformers;
                }
                return transformers;
            });
        });
    }
}
exports.AutomaticPropsType = AutomaticPropsType;
//# sourceMappingURL=automaticPropsType.js.map